/*
 * Created on 03.11.2005
 */
package de.unikassel.ir.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import junit.framework.TestCase;
import de.unikassel.ir.vsr.Corpus;
import de.unikassel.ir.vsr.CorpusImpl;
import de.unikassel.ir.vsr.Document;
import de.unikassel.ir.vsr.DocumentImpl;
import de.unikassel.ir.vsr.InvertedIndexImpl;
import de.unikassel.ir.vsr.TokenOccurrence;

public class IndexTest extends TestCase {
	private InvertedIndexImpl _index;

	public void setUp() {
		Corpus corpus = new CorpusImpl();

		File dir = new File("texte");
		for (File file : dir.listFiles()) {
			try {
				if (!file.isDirectory()) {  // CVS-Zeug �berspringen ;-)
					FileInputStream stream = new FileInputStream(file);
					Document doc = new DocumentImpl(file.getName());
					doc.read(stream);
					stream.close();
					corpus.addDocument(doc);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		_index = new InvertedIndexImpl(corpus);
	}
	
	public void testResults() {
		/* Resultate für die drei Terme "november", "shipbuilding", "sugarcane"
		 * vorgeben und mit den Resultaten des Index vergleichen
		 */ 
		Map<String, Double> novemberResult = new HashMap<String, Double>();
		Map<String, Double> shipbuildingResult = new HashMap<String, Double>();
		Map<String, Double> sugarcaneResult = new HashMap<String, Double>();
		
		Map<String, Map<String, Double>> results = new HashMap<String, Map<String, Double>>();
		
		novemberResult.put("Reut_8683.txt", 0.5245860368750322);
		novemberResult.put("Reut_3639.txt", 0.1748620122916774);
		novemberResult.put("Reut_10302.txt", 0.0874310061458387);
		novemberResult.put("Reut_4898.txt", 0.0874310061458387);
		novemberResult.put("Reut_5175.txt", 0.0874310061458387);
		novemberResult.put("Reut_11173.txt", 0.0874310061458387);
		novemberResult.put("Reut_4470.txt", 0.0874310061458387);
		results.put("november", novemberResult);
		
		shipbuildingResult.put("Reut_6541.txt", 0.2622930184375161);
		shipbuildingResult.put("Reut_1902.txt", 0.2622930184375161);
		shipbuildingResult.put("Reut_5818.txt", 0.1748620122916774);
		shipbuildingResult.put("Reut_11251.txt", 0.0874310061458387);
		shipbuildingResult.put("Reut_3046.txt", 0.0874310061458387);
		shipbuildingResult.put("Reut_4640.txt", 0.0874310061458387);
		shipbuildingResult.put("Reut_677.txt", 0.0874310061458387);
		results.put("shipbuilding", shipbuildingResult);
		
		sugarcaneResult.put("Reut_10306.txt", 0.27360110765801343);
		sugarcaneResult.put("Reut_11173.txt", 0.27360110765801343);
		sugarcaneResult.put("Reut_259.txt", 0.18240073843867563);
		sugarcaneResult.put("Reut_4630.txt", 0.18240073843867563);
		sugarcaneResult.put("Reut_11330.txt", 0.09120036921933782);
		sugarcaneResult.put("Reut_9069.txt", 0.09120036921933782);
		results.put("sugarcane", sugarcaneResult);

		// für alle drei Terme Resultate vergleichen
		for (String term : results.keySet()) {
			System.out.println("term: " + term);
			
			// erwartete Resultate für diesen Term holen
			Map<String, Double> expectedResults = results.get(term);

			double lastWeight = -1d;
			// Über alle Vorkommen gehen und Gewichte abgleichen
			for (Iterator<? extends TokenOccurrence> docWeights = _index
					.getWeightsForTerm(term); docWeights.hasNext();) {
				TokenOccurrence tokenOcc = docWeights.next();			
				Double weight = tokenOcc.getWeight();
				String docId = tokenOcc.getDocument().getId();
				
				// Gewichte prüfen gegen Referenzdaten
				Double expectedWeight = expectedResults.get(docId);
				assertEquals(expectedWeight, weight, 1E-7);
				
				// Sicherstellen, daß Gewichte absteigend sortiert sind,
				// außer beim ersten
				if (lastWeight > 0d) {
					assertTrue(weight - lastWeight < 1e-7);
				}
				lastWeight = weight;
			}
		}
	}
}
