/*
 * Created on 03.11.2005
 */
package de.unikassel.ir.vsr;

import java.util.Iterator;

/**
 * Interface für einen invertierten Index auf einer Menge von Dokumenten. Diese Menge
 * soll als {@link Corpus} im Konstruktor übergeben werden. Die Gewichte sollen
 * nach dem Einlesen über TF/IDF berechnet werden. 
 * Achtung: Die Termfrequenzen werden über den gesamten Korpus normiert (d.h. maxTf wird nicht für einzelne
 * Dokumente, sondern über den Korpus errechnet).
 * 
 * @author Christoph Schmitz
 * @author Beate Krause
 */
public interface InvertedIndex {
	
	/**
	 * Gib die Dokument-Term-Gewichte für diesen Term zurück,
	 * sortiert in absteigender Reihenfolge. 
	 * 
	 * @param term
	 * @return
	 */
	public Iterator<? extends TokenOccurrence> getWeightsForTerm(String term);
	
	/**
	 * Gibt den Index formatiert zurück in der Form
	 * 
	 * <pre>
	 * [
	 *   term -> [ docId -> tfidf,  docId -> tfidf, ... ]
	 *   term -> [ docId -> tfidf,  docId -> tfidf, ... ]
	 *   term -> [ docId -> tfidf,  docID -> tfidf, ... ]
	 * ]
	 * </pre>
	 */
	public String toString();
}
